package com.jpsoft.picc.modules.business.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONObject;
import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyUser;
import com.jpsoft.picc.modules.base.entity.InsuranceAgent;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinitionLimit;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.CompanyUserService;
import com.jpsoft.picc.modules.base.service.InsuranceAgentService;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import com.jpsoft.picc.modules.business.entity.*;
import com.jpsoft.picc.modules.business.service.*;
import com.jpsoft.picc.modules.common.config.PdfConfig;
import com.jpsoft.picc.modules.common.config.WeixinConfig;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.ItextPDFUtil;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.common.utils.WeixinUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.security.Policy;
import java.util.*;
import java.util.concurrent.Executor;

@RestController
@RequestMapping("/insurancePolicy")
@Api(description = "每月投保单")
public class InsurancePolicyController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsurancePolicyService insurancePolicyService;

    @Autowired
    private InsuranceAgentService insuranceAgentService;

    @Autowired
    private InsurancePolicyRecordService insurancePolicyRecordService;

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private ApplicationPolicyService applicationPolicyService;

    @Autowired
    private InsurancePolicyMemberService insurancePolicyMemberService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private CompanyService companyService;

    @Autowired
    private CompanyUserService companyUserService;

    @Autowired
    private WeixinConfig weixinConfig;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<InsurancePolicy> create(){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        InsurancePolicy insurancePolicy = new InsurancePolicy();

        msgResult.setData(insurancePolicy);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<InsurancePolicy> add(@RequestBody InsurancePolicy insurancePolicy,@RequestAttribute String subject){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            insurancePolicy.setId(UUID.randomUUID().toString());
            insurancePolicy.setDelFlag(false);
            insurancePolicy.setCreateBy(subject);
            insurancePolicy.setCreateTime(new Date());

            int affectCount = insurancePolicyService.insert(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<InsurancePolicy> edit(@PathVariable("id") String id){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

            if (insurancePolicy != null) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<InsurancePolicy> update(@RequestBody InsurancePolicy insurancePolicy,@RequestAttribute String subject){
        MessageResult<InsurancePolicy> msgResult = new MessageResult<>();

        try {
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());

            int affectCount = insurancePolicyService.update(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insurancePolicy);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);
            insurancePolicy.setDelFlag(true);
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());

            int affectCount = insurancePolicyService.update(insurancePolicy);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                InsurancePolicy insurancePolicy = insurancePolicyService.get(id);
                insurancePolicy.setDelFlag(true);
                insurancePolicy.setUpdateBy(subject);
                insurancePolicy.setUpdateTime(new Date());

                affectCount += insurancePolicyService.update(insurancePolicy);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<InsurancePolicy> page = insurancePolicyService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }


    @ApiOperation(value="读取附件")
    @RequestMapping(value = "attachmentList",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<List<Map<String,Object>>> attachmentList(String id, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<List<Map<String,Object>>> msgResult = new MessageResult<List<Map<String,Object>>>();

        InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

        String file = insurancePolicy.getInsurancePolicyFile();

        List<Map<String,Object>> fileList = new ArrayList<Map<String,Object>>();

        if(StringUtils.isNotEmpty(file)){
            String[] files = file.split(";");

            int i=1;

            for (String str:files) {
                Map<String,Object> map = new HashMap<String,Object>();
                map.put("index",i);
                map.put("file",str);
                String name = "";
                if(StringUtils.isNotEmpty(str)){
                    name = str.substring(str.lastIndexOf("/")+1);
                }
                map.put("name",name);
                fileList.add(map);
                i++;
            }
        }

        msgResult.setResult(true);
        msgResult.setData(fileList);

        return msgResult;
    }



    @ApiOperation(value="读取推广信息")
    @RequestMapping(value = "loadInsuranceAgent",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="id",value = "每月投保单ID",required = true,paramType = "query")
    })
    public MessageResult<InsuranceAgent> loadInsuranceAgent(String id, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<InsuranceAgent> msgResult = new MessageResult<>();

        InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

        String agentId = insurancePolicy.getAgentId();

        InsuranceAgent insuranceAgent = insuranceAgentService.get(agentId);

        msgResult.setResult(true);
        msgResult.setData(insuranceAgent);

        return msgResult;
    }


    @ApiOperation(value="人才超市初审")
    @RequestMapping(value = "firstApproval",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> firstApproval(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){
        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PolicyStatus policyStatus;

            if (processStatus.equals("正常")) {
                //30
                policyStatus = PolicyStatus.PendingRetrial;
            } else {
                //15
                policyStatus = PolicyStatus.Back;
            }

            int affectCount = approval(applicationId, policyId, processStatus, opinion, policyStatus.getValue() + "", subject);

            if (affectCount > 0) {
                //修改花名册人员状态为初审
                check(policyId, "1", subject);
            }

            //发送微信通知
            sendWeixinMessage(applicationId, policyStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount);
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    private void sendWeixinMessage(String applicationId, PolicyStatus policyStatus) {
        InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);
        Company company = companyService.get(insuranceApplication.getCompanyId());
        CompanyUser companyUser = companyUserService.findByCompanyId(company.getId());

        if (companyUser!=null && StringUtils.isNotEmpty(companyUser.getOpenId())){
            JSONObject jsonObject = new JSONObject();

            JSONObject userNameObj = new JSONObject();
            userNameObj.put("value",companyUser.getUserName());

            jsonObject.put("userName",userNameObj);

            JSONObject statusObj = new JSONObject();
            statusObj.put("value",policyStatus.getText());

            jsonObject.put("status",statusObj);

            new Thread(()->WeixinUtil.sendTemplateMessage(weixinConfig,
                    companyUser.getOpenId(),
                    weixinConfig.getSendTmplId1(), jsonObject)
            ).start();
        }
    }

    @ApiOperation(value="PICC复审")
    @RequestMapping(value = "secondApproval",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> secondApproval(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PolicyStatus policyStatus;

            if (processStatus.equals("正常")) {
                //40
                InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

                if (insurancePolicy != null) {
                    int no = insurancePolicy.getNo();
                    if (no > 1) {
                        //待制单
                        policyStatus = PolicyStatus.PendingMakePolicy;
                    } else {
                        //待缴费
                        policyStatus = PolicyStatus.PendingPay;
                    }
                } else {
                    throw new Exception("下月投保单不存在");
                }
            } else {
                //20
                policyStatus = PolicyStatus.PendingTrial;
            }

            int affectCount = approval(applicationId, policyId, processStatus, opinion, policyStatus.getValue() + "", subject);

            if (affectCount > 0) {
                //修改花名册人员状态为初审
                check(policyId, "2", subject);
            }

            //发送微信通知
            sendWeixinMessage(applicationId, policyStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount);
        }
        catch (Exception ex){
            logger.error(ex.getMessage(),ex);
            msgResult.setMessage(ex.getMessage());
            msgResult.setResult(false);
        }

        return msgResult;
    }


    @ApiOperation(value="PICC出单")
    @RequestMapping(value = "issued",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query"),
            @ApiImplicitParam(name="insurancePolicyFile",value = "投保单附件",required = true,paramType = "query")
    })
    public MessageResult<Integer> issued(String applicationId,String policyId,String processStatus,String opinion,String insurancePolicyFile,@RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
//        String status ="";
            PolicyStatus policyStatus;

            if (processStatus.equals("正常")) {
                //70
                policyStatus = PolicyStatus.SendOutPolicy;
            } else {
                //50
                policyStatus = PolicyStatus.PendingMakePolicy;
            }

            if (StringUtils.isNotEmpty(insurancePolicyFile)) {
                InsurancePolicy insurancePolicy = insurancePolicyService.get(policyId);

                insurancePolicy.setInsurancePolicyFile(insurancePolicyFile);

                insurancePolicyService.update(insurancePolicy);
            }

            int affectCount = approval(applicationId, policyId, processStatus, opinion, policyStatus.getValue() + "", subject);

            boolean flag = false;
            String msg = "";

            if (affectCount > 0) {
                if (processStatus.equals("正常")) {
                    //生成下月投保单
                    generatePolicyInTrans(applicationId, policyId, subject);

                    //发送微信通知
                    sendWeixinMessage(applicationId, policyStatus);
                } else {
                    flag = true;
                }
            }

            msgResult.setResult(flag);
            msgResult.setData(affectCount);
            msgResult.setMessage(msg);
        }
        catch (Exception ex) {
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }



    @ApiOperation(value="企业缴费")
    @RequestMapping(value = "pay",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query")
    })
    public MessageResult<Integer> pay(String applicationId,String policyId,String processStatus,String opinion, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        String status ="";

        if(processStatus.equals("正常")){
            //50
            status = String.valueOf(PolicyStatus.PendingMakePolicy.getValue());
        }
        else{
            //30
            status = String.valueOf(PolicyStatus.PendingRetrial.getValue());
        }

        int affectCount = approval(applicationId,policyId,processStatus,opinion,status ,subject);

        msgResult.setResult(true);
        msgResult.setData(affectCount);

        return msgResult;
    }



    @ApiOperation(value="人才超市与PICC转账")
    @RequestMapping(value = "transfer",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name="applicationId",value = "投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="policyId",value = "每月投保单ID",required = true,paramType = "query"),
            @ApiImplicitParam(name="accountBank",value = "入账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="issuingBank",value = "出账银行",required = true,paramType = "query"),
            @ApiImplicitParam(name="singlNumber",value = "单号",required = true,paramType = "query"),
            @ApiImplicitParam(name="processStatus",value = "流程审核状态（正常/回退）",required = true,paramType = "query"),
            @ApiImplicitParam(name="opinion",value = "意见",required = true,paramType = "query"),
            @ApiImplicitParam(name="agentId",value = "保险代理人ID",required = true,paramType = "query")
    })
    public MessageResult<Integer> transfer(String applicationId,String policyId,
                                           String accountBank,String issuingBank,String singlNumber,
                                           String processStatus,String opinion,String agentId, @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
//        String status ="";
            PolicyStatus policyStatus;

            if (processStatus.equals("正常")) {
                //60
                policyStatus = PolicyStatus.PendingOutPolicy;
            } else {
                //40
                policyStatus = PolicyStatus.PendingPay;
            }

            InsurancePolicy item = insurancePolicyService.get(policyId);

            item.setAccountBank(accountBank);
            item.setIssuingBank(issuingBank);
            item.setSinglNumber(singlNumber);
            item.setTransferTime(new Date());
            item.setAgentId(agentId);

            int affectCount = insurancePolicyService.update(item);

            int affectCount1 = 0;

            if (affectCount > 0) {
                affectCount1 = approval(applicationId, policyId, processStatus, opinion, policyStatus.getValue() + "", subject);
            }

            //发送微信通知
            sendWeixinMessage(applicationId, policyStatus);

            msgResult.setResult(true);
            msgResult.setData(affectCount1);
        }
        catch (Exception ex){
            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    private Integer approval(String applicationId,String id,String processStatus,String opinion,String status,String subject){

        int affectCount = 0;

        InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

        List<InsurancePolicy> insurancePolicyList = applicationPolicyService.findNewPolicyByApplicationId(applicationId);

        if(insurancePolicyList.size()>0){
            int no = insurancePolicyList.get(0).getNo();

            //如果每月投保单为第一份，就修改投保单状态
            if(no == 1){
                insuranceApplication.setStatus(status);
                insuranceApplicationService.update(insuranceApplication);
            }

            InsurancePolicy insurancePolicy = insurancePolicyService.get(id);

            insurancePolicy.setProcessStatus(processStatus);
            insurancePolicy.setStatus(status);
            insurancePolicy.setUpdateBy(subject);
            insurancePolicy.setUpdateTime(new Date());

            int affectCount1 = insurancePolicyService.update(insurancePolicy);

            if(affectCount1 > 0){
                InsurancePolicyRecord insurancePolicyRecord = new InsurancePolicyRecord();
                insurancePolicyRecord.setPolicyId(insurancePolicy.getId());
                insurancePolicyRecord.setStatus(status);
                insurancePolicyRecord.setOpinion(opinion);
                insurancePolicyRecord.setProcessStatus(processStatus);
                insurancePolicyRecord.setId(UUID.randomUUID().toString());
                insurancePolicyRecord.setDelFlag(false);
                insurancePolicyRecord.setCreateBy(subject);
                insurancePolicyRecord.setCreateTime(new Date());

                affectCount = insurancePolicyRecordService.insert(insurancePolicyRecord);
            }
        }

        return affectCount;
    }





    @Transactional(rollbackFor = Exception.class)
    private void generatePolicyInTrans(String applicationId, String policyId, String subject) throws Exception {
        //添加下月投保单
        InsurancePolicy srcPolicy = insurancePolicyService.get(policyId);
        srcPolicy.setStatus(PolicyStatus.SendOutPolicy.getValue() + "");
        srcPolicy.setUpdateTime(new Date());

        InsuranceApplication insuranceApplication = insuranceApplicationService.get(applicationId);

        insurancePolicyService.update(srcPolicy);

        InsurancePolicy destPolicy = new InsurancePolicy();

        PojoUtils.map(srcPolicy,destPolicy);

        DateTime startTime = new DateTime(srcPolicy.getEndTime());

        destPolicy.setStartTime(startTime.toDate());
        destPolicy.setEndTime(startTime.plusMonths(1).toDate());

        //已经过期则不生成下月投保单
        if (destPolicy.getEndTime().after(insuranceApplication.getEndTime())){
            return;
        }

        destPolicy.setEffectiveDate(startTime.toString("yyyyMM"));

        boolean exist = insurancePolicyService.existByApplicationIdAndEffectiveDate(applicationId,destPolicy.getEffectiveDate());

        if (exist){
            throw new Exception(destPolicy.getEffectiveDate() + "投保单已存在!");
        }

        destPolicy.setNo(srcPolicy.getNo()+1);
        destPolicy.setId(UUID.randomUUID().toString());
        destPolicy.setStatus(PolicyStatus.Draft.getValue() + "");
        destPolicy.setCreateBy(subject);
        destPolicy.setCreateTime(new Date());
        destPolicy.setUpdateBy(subject);
        destPolicy.setUpdateTime(new Date());

        insurancePolicyService.insert(destPolicy);

        //关联投保单及每月投保单
        ApplicationPolicy applicationPolicy = new ApplicationPolicy();
        applicationPolicy.setId(UUID.randomUUID().toString());
        applicationPolicy.setApplicationId(applicationId);
        applicationPolicy.setPolicyId(destPolicy.getId());

        applicationPolicyService.insert(applicationPolicy);

        //关联参保人员
        List<InsurancePolicyMember> memberList = insurancePolicyMemberService.findByPolicyId(policyId);

        for (InsurancePolicyMember srcMember : memberList) {
            InsurancePolicyMember destMember = new InsurancePolicyMember();

            destMember.setId(UUID.randomUUID().toString());
            destMember.setPolicyId(destPolicy.getId());
            destMember.setMemberId(srcMember.getMemberId());
            destMember.setDelFlag(false);
            destMember.setCreateBy(subject);
            destMember.setCreateTime(new Date());
            destMember.setStatus(srcMember.getStatus());

            insurancePolicyMemberService.insert(destMember);
        }
    }

    private void check(String policyId,String status,String subject){
        List<InsurancePolicyMember> insurancePolicyMemberList = insurancePolicyMemberService.findByPolicyId(policyId);
        for (InsurancePolicyMember insurancePolicyMember:insurancePolicyMemberList) {
            String statusMember = insurancePolicyMember.getStatus();
            if(StringUtils.isEmpty(statusMember)||statusMember.equals("0")){
                insurancePolicyMember.setStatus(status);
                insurancePolicyMember.setUpdateBy(subject);
                insurancePolicyMember.setUpdateTime(new Date());
                insurancePolicyMemberService.update(insurancePolicyMember);
            }
            else if(statusMember.equals("1")&&status.equals("2")){
                insurancePolicyMember.setStatus(status);
                insurancePolicyMember.setUpdateBy(subject);
                insurancePolicyMember.setUpdateTime(new Date());
                insurancePolicyMemberService.update(insurancePolicyMember);
            }
        }
    }
}
