package com.jpsoft.picc.modules.common.utils;

import cn.hutool.core.date.DateTime;
import com.aliyun.oss.HttpMethod;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.GeneratePresignedUrlRequest;
import com.aliyun.oss.model.OSSObject;
import com.aliyun.oss.model.PutObjectResult;
import com.jpsoft.picc.modules.common.config.OSSConfig;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.expression.DateTimeLiteralExpression;
import org.apache.commons.lang3.StringUtils;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

@Slf4j
public class OSSUtil {
    public static String upload(OSSConfig ossConfig,
                                   String subFolder, String fileName,
                                   InputStream fileInputStream) {
        Date now = new Date();

        Calendar cal = Calendar.getInstance();
        cal.setTime(now);

        String savePath = ossConfig.getObjectPre();

        if (!subFolder.startsWith("/")) {
            savePath += "/";
        }

        savePath += subFolder;

        savePath = savePath + "/" + cal.get(Calendar.YEAR) + "/" + (cal.get(Calendar.MONTH) + 1) + "/";

        OSS ossClient = new OSSClientBuilder().build(ossConfig.getEndpoint(), ossConfig.getAccessKeyId(), ossConfig.getAccessKeySecret());

        String retFileUrl = savePath + fileName;

        if(ossClient.doesObjectExist(ossConfig.getBucketName(), retFileUrl)){
            String ext = fileName.substring(fileName.indexOf("."));

            String newFileName = DateTime.now().toString("yyyyMMddHHmmssSSS") + ext;

            retFileUrl = savePath + newFileName;
        }

        // 上传文件流
        PutObjectResult result = ossClient.putObject(ossConfig.getBucketName(), retFileUrl, fileInputStream);

        // 关闭OSSClient
        ossClient.shutdown();

        return ossConfig.getUrlPrefix() + "/" + retFileUrl;
    }

    public static void batchDownload(List<Map<String,String>> fileList, OutputStream output){
        BufferedInputStream bis = null;

        try{
            ZipOutputStream zos = new ZipOutputStream(output);

            for (Map<String,String> map : fileList) {
                String fileUrl = map.get("fileUrl");

                URL url = new URL(fileUrl);
                URLConnection conn = url.openConnection();
                InputStream inputStream = conn.getInputStream();

                byte[] buffs = new byte[1024 * 10];

                String fileName = map.get("fileName");

                if (StringUtils.isEmpty(fileName)){
                    fileName = fileUrl;
                }

                if (fileName.indexOf("?")!=-1){
                    fileName = fileName.substring(0,fileName.indexOf("?"));
                }

                fileName = fileName.substring(fileName.lastIndexOf("/") + 1);

                String zipFile = fileName;

                ZipEntry zipEntry = new ZipEntry(zipFile);
                zos.putNextEntry(zipEntry);
                bis = new BufferedInputStream(inputStream, 1024 * 10);

                int read;
                while ((read = bis.read(buffs, 0, 1024 * 10)) != -1) {
                    zos.write(buffs, 0, read);
                }

                inputStream.close();
            }

            zos.close();
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);
        }
        finally {
            if(bis!=null){
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    public static void presignedDownload(OSSConfig ossConfig, List<String> fileList, OutputStream output){
        BufferedInputStream bis = null;

        try{
            ZipOutputStream zos = new ZipOutputStream(output);

            OSS ossClient = new OSSClientBuilder().build(ossConfig.getEndpoint(), ossConfig.getAccessKeyId(), ossConfig.getAccessKeySecret());

            for (String filePath : fileList) {
                Date expiration = new Date(System.currentTimeMillis() + 3600 * 1000);

                if (filePath.startsWith(ossConfig.getUrlPrefix())){
                    filePath = filePath.substring(ossConfig.getUrlPrefix().length());
                }

                if(filePath.indexOf("?")!=-1){
                    filePath = filePath.substring(0,filePath.indexOf("?"));
                }

                if (filePath.startsWith("/")){
                    filePath = filePath.substring(1);
                }

                GeneratePresignedUrlRequest request = new GeneratePresignedUrlRequest(ossConfig.getBucketName(), filePath, HttpMethod.GET);

                // 设置过期时间。
                request.setExpiration(expiration);

                //设置缩放
                request.setProcess("image/resize,l_1024,limit_1");

                // 生成签名URL（HTTP GET请求）。
                URL signedUrl = ossClient.generatePresignedUrl(request);

                // 使用签名URL发送请求。
                OSSObject ossObject = ossClient.getObject(signedUrl, new HashMap<>());

                if(ossObject!=null) {
                    InputStream inputStream = ossObject.getObjectContent();
                    byte[] buffs = new byte[1024 * 10];
                    String fileName = filePath.substring(filePath.lastIndexOf("/") + 1);
                    String zipFile = fileName;

                    ZipEntry zipEntry = new ZipEntry(zipFile);
                    zos.putNextEntry(zipEntry);
                    bis = new BufferedInputStream(inputStream, 1024 * 10);

                    int read;
                    while ((read = bis.read(buffs, 0, 1024 * 10)) != -1) {
                        zos.write(buffs, 0, read);
                    }

                    ossObject.close();
                }
            }

            zos.close();
            ossClient.shutdown();
        }
        catch(Exception ex){
            log.error(ex.getMessage(),ex);
        }
        finally {
            if(bis!=null){
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
