package com.jpsoft.picc.modules.auth.controller;

import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.CompanyUser;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.CompanyUserService;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import com.jpsoft.picc.modules.pub.dto.CompanyInfoDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.UUID;

@CrossOrigin
@Api(description="企业信息管理")
@RestController
@RequestMapping("/auth/company")
public class CompanyController {
    @Autowired
    private CompanyService companyService;

    @Autowired
    private CompanyUserService companyUserService;

    @GetMapping(value="detail")
    @ApiOperation(value = "获取企业信息")
    public MessageResult<CompanyInfoDTO> detail(HttpServletRequest request){
        MessageResult<CompanyInfoDTO> messageResult = new MessageResult<>();

        try {
            CompanyInfoDTO companyInfoDTO = new CompanyInfoDTO();

            //todo
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            Company company = null;

            if (companyUser!=null && StringUtils.isNotEmpty(companyUser.getCompanyId())) {
                company = companyService.get(companyUser.getCompanyId());
            }

            if(company!=null) {
                PojoUtils.map(company, companyInfoDTO);
                messageResult.setData(companyInfoDTO);
            }
            else{
                messageResult.setData(companyInfoDTO);
            }

            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping(value="save")
    @ApiOperation(value = "保存企业信息")
    public MessageResult<String> save(CompanyInfoDTO companyInfoDTO,HttpServletRequest request){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            //todo
            AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

            Company company = null;

            if (!StringUtils.isEmpty(companyInfoDTO.getId())){
                company = companyService.get(companyInfoDTO.getId());
            }

            if (company==null) {
                company = new Company();
                PojoUtils.map(companyInfoDTO, company);

                company.setId(UUID.randomUUID().toString());
                company.setCreateTime(new Date());
                company.setCreateBy(principal.getName());
                company.setDelFlag(false);

                companyService.insert(company);

                messageResult.setData(company.getId());
            }
            else {
                PojoUtils.map(companyInfoDTO, company);
                company.setUpdateTime(new Date());
                company.setUpdateBy(principal.getName());

                companyService.update(company);

                messageResult.setData(company.getId());
            }

            CompanyUser companyUser = companyUserService.findByUserName(principal.getName());
            companyUser.setCompanyId(company.getId());
            companyUser.setUpdateTime(new Date());
            companyUserService.update(companyUser);

            messageResult.setResult(true);
        }
        catch (Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
